/************************************************************************
 * This macro assigns both dimensions of the 1991 canadian         ******
 * deprivation index to any administrative file with a postal      ******
 * code variable and, optionally, a census subdivision variable.   ******
 * Four versions, canadian, regional, metropolitan and by zone     ******
 * zone, are assigned. A simplified version of the PCCF+, a        ******
 * Statistics Canada product, is used to link the enumeration      ******
 * area with the postal code.                                      ******
 *                                                                 ******
 * Macro parameters:                                               ******
 *  - IN: SAS input file on which the user wants to add the        ******
 *        material and social deprivation index (4 versions)       ******
 *  - PCODE: six-character postal code variable name               ******
 *  - MUNIC: census subdivison variable name. By default,          ******
 *           0 = the index is assigned with the postal code only   ******
 *  - OUT: SAS output file name,                                   ******
 *                                                                 ******
 * Authors: Philippe Gamache - philippe.gamache@inspq.qc.ca        ******
 * Last modified: November 27th                                    ******
 * Modifications: English version finalized                        ******
 ************************************************************************/


filename fccp "X:\Your Directory\fccp.txt";
filename munic "X:\Your Directory\munic.txt";
libname i1991 "X:\Your Directory";

data i1991.fccp;
infile fccp;
input  munic 1-12 pcode $ 13-18 @19 propcp 5.2 propmunic 5.2 quintmat 33 quintsoc 34 centmat 35-37 centsoc 38-40
       quintmatRC 41 quintsocRC 42 quintmatZONE 43 quintsocZONE 44 quintmatRMR 45 quintsocRMR 46 zone 47;
run;

data i1991.munic;
  infile munic;
  input munic 1-7 mun96 8-14 mun01 15-21;
run;

%macro presence(var=);
%global presence;
%let dsid=%sysfunc(open(fichin,i));
    %if &dsid>0 %then
       %do;
	     %let presence=%sysfunc(varnum(&dsid,&var));
	   %end;
%put &presence;
%let rc = %sysfunc(close(&dsid));
%mend presence;

%macro assignationInd_91_eng(in,pcode,munic,out);

data fccp;
   set i1991.fccp;
   nuniq = _n_;
run;

  data fichin;
    %if &munic ne 0 %then %do; length munic 8.; %end;
    set &in;
    numero=_n_;
	nba=ranuni(766);

	pcode=compress(&pcode);
			%if &munic ne 0 %then 
     %do; 
       if substr(pcode,1,1) in ("A") then munic=1000000+mod(&munic,1000000);
       else if substr(pcode,1,1) in ("B") then munic=1200000+mod(&munic,1200000);
       else if substr(pcode,1,1) in ("C") then munic=1100000+mod(&munic,1100000);
       else if substr(pcode,1,1) in ("E") then munic=1300000+mod(&munic,1300000);
       else if substr(pcode,1,1) in ("G","H","J") then munic=2400000+mod(&munic,2400000);
       else if substr(pcode,1,1) in ("K","L","M","N","P") then munic=3500000+mod(&munic,3500000);
       else if substr(pcode,1,1) in ("R") then munic=4600000+mod(&munic,4600000);
       else if substr(pcode,1,1) in ("S") then munic=4700000+mod(&munic,4700000);
       else if substr(pcode,1,1) in ("T") then munic=4800000+mod(&munic,4800000);
       else if substr(pcode,1,1) in ("V") then munic=5900000+mod(&munic,5900000);
       else if substr(pcode,1,1) in ("X") then munic=6100000+mod(&munic,6100000);
       else if substr(pcode,1,1) in ("Y") then munic=6000000+mod(&munic,6000000);
     %end;
   run;

%presence(var=quintmat);

%if &presence ne 0 %then %do;
   data fichin;
   set fichin;
   where zone in (.,0);
   run;
%end;

data fichin;
  set fichin;
  keep numero nba pcode %if &munic ne 0 %then %do; munic %end;;
run;

%if &munic ne 0 %then %do;


proc sql;
     create table test as
     select fichin.pcode, fichin.munic, numero, nba, fccp.propmunic, fccp.nuniq,
	        fccp.quintmat, fccp.quintsoc, fccp.centmat, fccp.centsoc, fccp.zone,
			fccp.quintmatRC, fccp.quintsocRC, fccp.quintmatZONE, fccp.quintsocZONE, fccp.quintmatRMR, fccp.quintsocRMR
     from fichin, fccp
     where fichin.pcode=fccp.pcode and fichin.munic=fccp.munic
	 order by numero,nuniq; 
    quit;

  proc sort data=test;
    by pcode munic numero;
   run;

  data test2;
    set test;
    by pcode munic numero;
	retain cumul find direct;
	if first.numero then do; find=0; cumul=0; 
                             if last.numero then direct=1; else direct=0;
                         end;
	if find = 0 then do;
	   cumul=cumul+(propmunic/100);
	   if nba < cumul then do; find=1; output; end;
      end;

   run; 

  proc sort data=test2;
    by numero;
   run;

  proc sort data=fichin;
    by numero;
   run; 

  data appmunic1 fichin;
    merge fichin(in=in1) test2(in=in2);
	by numero;
	if in1*in2 then output appmunic1;
	else if in1 then output fichin;
   run;

   %end;

   proc sql;
     create table test as
     select fichin.pcode, numero, nba, fccp.propcp, fccp.nuniq,
	        fccp.quintmat, fccp.quintsoc, fccp.centmat, fccp.centsoc, fccp.zone,
			fccp.quintmatRC, fccp.quintsocRC, fccp.quintmatZONE, fccp.quintsocZONE, fccp.quintmatRMR, fccp.quintsocRMR
     from fichin, fccp
     where fichin.pcode=fccp.pcode
	 order by numero,nuniq; 
    quit;

  proc sort data=test;
    by pcode numero;
   run;


  data test2;
    set test;
    by pcode numero;
	retain cumul find direct;
	if first.numero then do; find=0; cumul=0; 
                             if last.numero then direct=1; else direct=0;
                         end;
	if find = 0 then do;
	   cumul=cumul+(propcp/100);
	   if nba < cumul then do; find=1; output; end;
      end;

   run; 

  proc sort data=test2;
    by numero;
   run;

  proc sort data=fichin;
    by numero;
   run; 

  data appcp1 noapp1;
    merge fichin(in=in1) test2(in=in2);
	by numero;
	if in1*in2 then output appcp1;
	else if in1 then output noapp1;
   run;

data tout;
    set %if &munic ne 0 %then %do; 
                                appmunic1(in=in1)
                              %end;
	appcp1(in=in2) noapp1(in=in3);

	 if in3=1 then do; quintmat = 0; quintsoc = 0; centmat = 0; centsoc = 0; zone = 0;
     end;

     if quintmat=0 then groupe=3;
	 else if quintmat=. then groupe=2;
	 else groupe=1;

	keep numero centmat centsoc quintmat quintsoc quintmatRC quintsocRC quintmatZONE quintsocZONE quintmatRMR quintsocRMR zone groupe;
   run;

  proc sort; by numero; run; 

  data fichin;
     set &in;
	 numero=_n_;
    run;

  data &out;
    merge fichin tout;
	by numero;
	DATE = '20092711';
	group = groupe;quintmatCR = quintmatRC;quintsocCR = quintsocRC;quintmatCMA = quintmatRMR;quintsocCMA = quintsocRMR;
	format group fgroupe.;
	drop numero groupe quintmatRC quintsocRC quintmatRMR quintsocRMR;
   run;

quit;
%mend assignationInd_91_eng;

proc format;
  value fgroupe 1="Successful assignment"
  		        2="Linked to an EA without an index"
				3="Unsuccessful assignment - Postal code not found";
 run;


  /***************************************************************
 * Example: %assignationInd_91_eng(birth91, pc, cen_div, birth91v2);  ******
 * - The input file is WORK.birth91                         ******
 * - The postal code variable is PC.                        ******
 * - The census subdivision variable is cen_div.            ******
 *   (N.B.: is it important to put 0 if you don't want to   ******
 *   assign the index with the census subdivision variable. ******
 * - The output file is WORK.birth91V2 (identical from the  ******
 *   input file, where the deprivation index (quintiles,    ******
 *   centiles),  were added                                 ******
 * - NOTE: The results will differ if the order of the      ******
 *   data in the input file is not the same.                ******
 *****************************************************************/ 

 libname toto "X:\Your Directory";

%assignationInd_91_eng(in=birth91,pcode=pc,munic=cen_div,out=birth91v2);
